@@ -3,32 +3,30 @@ getTargets <- function(data, effects)
 {
     f <- unpackData(data)
     effects <- effects[effects$include,]
-    ##dyn.load(dllpath)
     ##
-    ## dyn.load('d:/sienasvn/siena/src/RSiena.dll')
-    pData <- .Call('setupData', PACKAGE="RSiena",
+    pData <- .Call('setupData', PACKAGE=pkgname,
                    list(as.integer(f$observations)),
                    list(f$nodeSets))
     ## register a finalizer
     ans <- reg.finalizer(pData, clearData, onexit = FALSE)
-    ans<- .Call('OneMode', PACKAGE="RSiena",
+    ans<- .Call('OneMode', PACKAGE=pkgname,
                 pData, list(f$nets))
-    ans<- .Call('Behavior', PACKAGE="RSiena", pData,
+    ans<- .Call('Behavior', PACKAGE=pkgname, pData,
                list(f$behavs))
-    ans<-.Call('ConstantCovariates', PACKAGE="RSiena",
+    ans<-.Call('ConstantCovariates', PACKAGE=pkgname,
                pData, list(f$cCovars))
-    ans<-.Call('ChangingCovariates',PACKAGE="RSiena",
+    ans<-.Call('ChangingCovariates',PACKAGE=pkgname,
                pData,list(f$vCovars))
-    ans<-.Call('DyadicCovariates',PACKAGE="RSiena",
+    ans<-.Call('DyadicCovariates',PACKAGE=pkgname,
                pData,list(f$dycCovars))
-    ans<-.Call('ChangingDyadicCovariates',PACKAGE="RSiena",
+    ans<-.Call('ChangingDyadicCovariates',PACKAGE=pkgname,
                pData, list(f$dyvCovars))
     storage.mode(effects$parm) <- 'integer'
     storage.mode(effects$group) <- 'integer'
     storage.mode(effects$period) <- 'integer'
     effects$effectPtr <- NA
     myeffects <- split(effects, effects$name)
-    ans<- .Call('effects', PACKAGE="RSiena",
+    ans<- .Call('effects', PACKAGE=pkgname,
                 pData, myeffects)
     pModel <- ans[[1]][[1]]
         for (i in 1:length(ans[[2]])) ## ans[[2]] is a list of lists of
@@ -38,7 +36,7 @@ getTargets <- function(data, effects)
             effectPtr <- ans[[2]][[i]]
             myeffects[[i]]$effectPtr <- effectPtr
         }
-    ans <- .Call('getTargets', PACKAGE="RSiena",
+    ans <- .Call('getTargets', PACKAGE=pkgname,
                  pData, pModel, myeffects)
     ans
 }
