@@ -56,8 +56,7 @@ robmon <- function(z, x, useCluster, nbrNodes, initC, clusterString,
         clusterCall(cl, storeinFRANstore,  FRANstore())
         if (initC)
         {
-            ans <-  clusterCall(cl, usesim, z, x,
-                                INIT=FALSE, initC = initC)
+            clusterCall(cl, usesim, z, x, INIT=FALSE, initC = initC)
         }
         z$cl <- cl
     }
