@@ -11,7 +11,7 @@
 ## ****************************************************************************/
 
 ##@siena07 siena07
-siena07<- function(x, batch = FALSE, verbose = FALSE, silent=FALSE,
+siena07 <- function(x, batch = FALSE, verbose = FALSE, silent=FALSE,
                    useCluster = FALSE, nbrNodes = 2, initC=TRUE,
                    clusterString=rep("localhost", nbrNodes), tt=NULL,
                    parallelTesting=FALSE, clusterIter=!x$maxlike, ...)
@@ -398,7 +398,7 @@ tkErrorMessage <- function()
 ##@errorHandler Miscellaneous Not used
 errorHandler <- function()
 {
-    opts <- options()
+    ##  opts <- options()
     if (!is.batch())
     {
         options(show.error.messages=FALSE)
