@@ -34,16 +34,24 @@ public:
 	virtual double calculateContribution(int alter) const;
 
 protected:
-	virtual double statistic(const Network * pSummationTieNetwork) const;
+	virtual void initializeStatisticCalculation();
+	virtual void cleanupStatisticCalculation();
+	virtual double tieStatistic(int alter);
 
 private:
 	void markInvalidActors(IncidentTieIterator iter,
-		int * flag,
-		int markValue,
-		int & validActorCount) const;
+		int & validActorCount);
 
 	// The centering constant b_0.
 	double lbalanceMean;
+
+	// An indicator array for invalid actors (used in statistic calculations)
+	// Invariants:
+	// A: lflag[i] <= lround for all actors
+	// B: lflag[i] == lround for invalid actors
+
+	int * lflag;
+	int lround;
 };
 
 }
