@@ -53,7 +53,11 @@ public:
 	virtual double evaluationStatistic(double * currentValues);
 	virtual double endowmentStatistic(const int * difference,
 		double * currentValues);
+	virtual double creationStatistic(int * difference,
+		double * currentValues);
 	virtual double egoStatistic(int ego, double * currentValues);
+	virtual double egoEndowmentStatistic(int ego, const int * difference,
+		double * currentValues);
 
 protected:
 	int n() const;
