@@ -35,7 +35,7 @@ public:
 	virtual double calculateContribution(int alter) const;
 
 protected:
-	virtual double statistic(const Network * pSummationTieNetwork) const;
+	virtual double tieStatistic(int alter);
 
 private:
 	void calculateSums(int i, const Network * pNetwork, double * sums) const;
