@@ -27,7 +27,7 @@ namespace siena
 class MiniStep;
 class Data;
 class State;
-
+class MLSimulation;
 
 // ----------------------------------------------------------------------------
 // Section: Class definition
@@ -53,9 +53,14 @@ public:
 	void clear();
 	void insertBefore(MiniStep * pNewMiniStep, MiniStep * pExistingMiniStep);
 	void remove(MiniStep * pMiniStep);
-	void connect(int period);
+	void connect(int period, MLSimulation * pMLSimulation);
 	void onReciprocalRateChange(const MiniStep * pMiniStep, double newValue);
 	void changeInitialState(const MiniStep * pMiniStep);
+	void recreateInitialState();
+	void createInitialStateDifferences();
+	void addInitialStateDifference(MiniStep * pMiniStep);
+	void addEndStateDifference(MiniStep * pMiniStep);
+	void clearEndStateDifferences();
 
 	// Accessors
 
@@ -71,7 +76,11 @@ public:
 	int missingBehaviorMiniStepCount() const;
 	double mu() const;
 	double sigma2() const;
+	double finalReciprocalRate() const;
+	void finalReciprocalRate(double value);
 	void printConsecutiveCancelingPairs() const;
+	const vector <MiniStep *> & rEndStateDifferences() const;
+	const vector <MiniStep *> & rInitialStateDifferences() const;
 
 	// Intervals
 
@@ -84,6 +93,11 @@ public:
 	MiniStep * nextMiniStepForOption(const Option & rOption,
 		const MiniStep * pFirstMiniStep) const;
 
+	// Same dyad related
+
+	MiniStep * pFirstMiniStepForLink(const MiniStep * pLinkMiniStep) const;
+	MiniStep * pLastMiniStepForLink(const MiniStep * pLinkMiniStep) const;
+
 	// Random draws
 
 	MiniStep * randomMiniStep() const;
@@ -118,6 +132,14 @@ private:
 	// The initial state of the variables (denoted y_init in the specification)
 	State * lpInitialState;
 
+	// The initial state of the variables stored as a vector of ministeps
+	// relative to data (to save space)
+	vector<MiniStep *> linitialStateDifferences;
+
+	// The final state of the variables stored as a vector of ministeps
+	// relative to data (to save space)
+	vector<MiniStep *> lendStateDifferences;
+
 	// Stores the ministeps in no particular order.
 	// The first (dummy) ministep is not stored in this vector.
 
@@ -141,6 +163,9 @@ private:
 	// Sum of squared reciprocal rates over all non-dummy ministeps.
 	double lsigma2;
 
+	// Final reciprocal rate when updating part por all of the probabilities.
+	double lfinalReciprocalRate;
+
 	// Maps each option to its first ministep in the chain (if any)
 	map<const Option, MiniStep *> lfirstMiniStepPerOption;
 };
